/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
/******************************************************************************
 *  config.c   read and process config files                                  *
 ******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "vector.h"
#define DIR 1
#define LAYER 2
#define FEATURE 3

extern char *inifile;
extern char *mystrsep( char **stringp, char *delim);
extern int GetPrivateProfileString(char *section, char *entry, char *def_val,
                            char *buffer, int len, char *filename);

#define NDEBUG

#define DEF_LAYERS "states,indian,msa,counties,mcd,con_dist,zip_code,places,censusb,cen_blkg,tract,miscell,water,shorelin,railroad,roads,majroads"

#define DEF_FEATURES "A10A11A14,A15A18,A63,A20A21A24,A25A8,A30A31A34,A35A38,A03A07A13A17A2327A33A37A43A47,A02A06A12A16A22A2632A36A42A46,A7*A64A65A5*,H00H10H1H13,H02H12H22H32H42,H7*,C**E**D2*,D0,D84D6*,D83,D85D81D82,D3*D41D42D4D50D52D53D54D55,D40D43,M00,M01"


static char *skip(char *p)
{
    while (*p) {
	if (*p++ == ':')
	    return p;
    }
    return p;
}

void LineErr(char *text)
{
    fprintf(stderr, "incorrect syntax: %s\n", text);
    exit(0);
}

void ReadColor(char *value, Config * sym)
{
    char line[32] = "-", fill[32] = "-", point[32] = "-";

    sscanf(value, "%[^;];%[^;];%s", line, fill, point);
    if (sscanf(line, "%hi,%hi,%hi,%hi", &sym->line.r, &sym->line.g, &sym->line.b, &sym->line.w) < 4) {
	if (strlen(line) == 0)
	    strcpy(line, "-");
	strcpy(sym->line.file, line);
    }
    if (sscanf(fill, "%hi,%hi,%hi", &sym->fill.r, &sym->fill.g, &sym->fill.b) < 3) {
	if (strlen(fill) == 0)
	    strcpy(fill, "-");
	strcpy(sym->fill.file, fill);
    }
    if (sscanf(point, "%hi,%hi,%hi,%hi", &sym->point.r, &sym->point.g, &sym->point.b, &sym->point.w) < 4) {
	if (strlen(point) == 0)
	    strcpy(point, "-");
	strcpy(sym->point.file, point);
    }
}

void do_dataset(char *value, MapStruct * ms, char *fn)
{
    int force, done, theme;
    unsigned long max;
    char *p, *t;
    char bufp[256];
    char set[256];

    sprintf(set, "dataset:%s", value);
    done = 0;

    strcpy(ms->datasets[ms->ndirs++].name, value);
    if (strstr(ms->forceoff, value))
	done = 1;
    if (strstr(ms->forceon, value))
	force = 1;
    else
	force = 0;
    if (strstr(ms->theme.level, value) && ms->theme.on)
	theme = 1;
    else
	theme = 0;

    GetPrivateProfileString(set, "label", "", bufp, 256, fn);
    if (*bufp)
	strcpy(ms->datasets[ms->ndirs - 1].label, bufp);

    GetPrivateProfileString(set, "scale", "", bufp, 256, fn);
    t = bufp;
    while ((p = mystrsep(&t, " ")) && !done) {

	assert(p != NULL);

	if (!strncmp(p, "-", 1))
	    max = -1;
	else if (!sscanf(p, "%lu", &max))
	    LineErr(p);

	p = skip(p);

	if (!done && (((unsigned long) ms->scale < (unsigned long) max) || force)) {
	    assert(p != NULL);
	    if (!strcmp(p, "X")) {
		if (!force)
		    done = 1;
	    } else {
		ReadColor(p, &ms->datasets[ms->ndirs - 1]);
		ms->datasets[ms->ndirs - 1].on = 1;
	    }
	}
	if ((unsigned long) ms->scale < (unsigned long) max)
	    done = 1;
    }
}

void do_layer(char *value, MapStruct * ms, char *fn)
{
    int force, done, theme;
    unsigned long max;
    char *p, *t;
    char bufp[256];
    char set[256];

    sprintf(set, "layer:%s", value);
    done = 0;

    strcpy(ms->layers[ms->nlyrs++].name, value);
    if (strstr(ms->forceoff, value))
	done = 1;
    if (strstr(ms->forceon, value))
	force = 1;
    else
	force = 0;
    if (strstr(ms->theme.level, value) && ms->theme.on)
	theme = 1;
    else
	theme = 0;

    GetPrivateProfileString(set, "label", "", bufp, 256, fn);
    if (*bufp)
	strcpy(ms->datasets[ms->nlyrs - 1].label, bufp);

    GetPrivateProfileString(set, "scale", "", bufp, 256, fn);
    t = bufp;
    while ((p = mystrsep(&t, " ")) && !done) {

	assert(p != NULL);

	if (!strncmp(p, "-", 1))
	    max = -1;
	else if (!sscanf(p, "%lu", &max))
	    LineErr(p);

	p = skip(p);

	if (!done && (((unsigned long) ms->scale < (unsigned long) max) || force)) {
	    assert(p != NULL);
	    if (!strcmp(p, "X")) {
		if (!force)
		    done = 1;
	    } else {
		ReadColor(p, &ms->layers[ms->nlyrs - 1]);
		ms->layers[ms->nlyrs - 1].on = 1;
	    }
	}
	if ((unsigned long) ms->scale < (unsigned long) max)
	    done = 1;
    }
}

void do_feature(char *value, MapStruct * ms, char *fn)
{
    int force, done, theme;
    unsigned long max;
    char *p, *t;
    char bufp[256];
    char set[256];

    sprintf(set, "feature:%s", value);
    done = 0;

    strcpy(ms->features[ms->nfeats++].name, value);
    if (strstr(ms->forceoff, value))
	done = 1;
    if (strstr(ms->forceon, value))
	force = 1;
    else
	force = 0;
    if (strstr(ms->theme.level, value) && ms->theme.on)
	theme = 1;
    else
	theme = 0;

    GetPrivateProfileString(set, "label", "", bufp, 256, fn);
    if (*bufp)
	strcpy(ms->features[ms->nfeats - 1].label, bufp);

    GetPrivateProfileString(set, "scale", "", bufp, 256, fn);
    t = bufp;
    while ((p = mystrsep(&t, " ")) && !done) {

	assert(p != NULL);

	if (!strncmp(p, "-", 1))
	    max = -1;
	else if (!sscanf(p, "%lu", &max))
	    LineErr(p);

	p = skip(p);

	if (!done && (((unsigned long) ms->scale < (unsigned long) max) || force)) {
	    assert(p != NULL);
	    if (!strcmp(p, "X")) {
		if (!force)
		    done = 1;
	    } else {
		ReadColor(p, &ms->features[ms->nfeats - 1]);
		ms->features[ms->nfeats - 1].on = 1;
	    }
	}
	if ((unsigned long) ms->scale < (unsigned long) max)
	    done = 1;
    }
}

void ReadConfig(MapStruct * ms, char *fn)
{
    char *p, *t;
    int done;
    char datasets[256];
    char layers[1024];
    char features[1024];

    done = 0;
    fn = inifile;

    GetPrivateProfileString("order", "datasets", "BACK,USA,GRID,*,CITIES",
			    datasets, 256, fn);
    GetPrivateProfileString("order", "layers", DEF_LAYERS,
			    layers, 1024, fn);
    GetPrivateProfileString("order", "features", DEF_FEATURES,
			    features, 1024, fn);

    t = datasets;
    while ((p = mystrsep(&t, ","))) {

	assert(p != NULL);

	do_dataset(p, ms, fn);
    }

    t = layers;
    while ((p = mystrsep(&t, ","))) {

	assert(p != NULL);

	do_layer(p, ms, fn);
    }

    t = features;
    while ((p = mystrsep(&t, ","))) {

	assert(p != NULL);

	do_feature(p, ms, fn);
    }
}
